import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * The <code>GUI</code> class is used to visualize the worldmodel of one 
 * special player. Also the tactical considerations are shown in a stream.
 *
 * @author <a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
 * @author <a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
 * @author <a href="mailto:wini.appl@epost.de">Wini Appl</a>
 */

public class GUI extends JFrame implements ItemListener
{

	//attributes
	/**The player[] object*/
	private Player[] player;
	/**The GUI object*/
	private Spielfeld sf;
 	/**Selected player*/
 	private int d=1;
	/**The combobox component*/
	private JComboBox selectedplayer;
	/**The textarea component*/
	private JTextArea daten;

	
	
	/**
	* Eventhandling for the window 
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	class MeinWindowAdapter extends WindowAdapter 
    {
        public void windowClosing(WindowEvent e)
        {
        	dispose();
        }
    }
    
    
    /** 
    * Eventhandling for the <code>JButton</code> "Beenden" is being capsuled here.<br/>
    * <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>,
    * <a href="mailto:wini.appl@epost.de">Wini Appl</a>
    *
    * @see GUI#GUI(Player#[] player)
    */
    class actionlistener implements ActionListener
    {
    	public void actionPerformed(ActionEvent ev)
    	{
      		String aktion = ev.getActionCommand();

      		if(aktion.equals("Beenden"))
      		{
      			dispose();
      			System.out.println("<GUI has been closed>");
      		}
        		  	
    	}
  	}
  	
  	
  	/**
  	* Here the Player chosen by <code>JPanel</code> "Beenden" is determined.
  	* Gets displayed on Tactics Display.<br/>
  	*
  	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>,
  	* <P><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
  	*
  	* @see GUI#GUI(Player[] player)
  	*/
  	public void itemStateChanged(ItemEvent ev)
  	{
  		d = 1;
  		for (int i=1;i<=11;i++)
  			{
  				if (ev.getItem().equals("Player "+i))
  				{	
  					d=i;
  					
  					RoboClient.transmitTactics(new String(":Monitoring P" + d + "::::"),0);	
  					break;
  				}
  			}

  	}
 		
 		
 	/**
 	*Returns the value of the selected item 
 	 <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
 	*/
 	
 	public int getPlayer()
  	{
	  	return d;
  	}
	
	/**
	* Constructor. Creates a new GUI instance
	* <P><b>Author: </b><a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
	*
	* @param player a <code>Player[]</code> array pointing on the 11 <code>Player</code> threads
	*/
	public GUI(Player[] player)
	{
		
		// title must stand on first place
		super("Visualisierung");
		
		this.player = player;
		
		//creates a new spielfeld and starts thread with startAnimation() method
		sf = new Spielfeld(this, player);
		sf.startAnimation();
		
		Container c = getContentPane();
		//Layout for the content pane of the JFrame
		c.setLayout(null);
		
		//new JPanel for JCombobox and JButton
		JPanel panel1 = new JPanel();
		panel1.setLayout(new FlowLayout());
		
		//JCombobox
		String selectedPlayer[] ={"Player 1", "Player 2", "Player 3", 
		"Player 4", "Player 5", "Player 6",
   		"Player 7", "Player 8", "Player 9", "Player 10", "Player 11"};
		JComboBox selectedplayer = new JComboBox (selectedPlayer);
		
		//JButton
		JButton beenden = new JButton("Beenden");
		
		//JTextArea
		daten = new JTextArea("Daten von:"+ getPlayer(), 8, 14);
		daten.setBackground(Color.black);
		daten.setForeground(Color.white);
		//turns on automatic linewrapping
		daten.setLineWrap(true);
		daten.setAutoscrolls(true);
	
		//Add button and combobox to panel
   		panel1.add(selectedplayer);
		panel1.add(beenden); 
	
		//set bounds for components because of null layout
		panel1.setBounds(142,0,650,30);
		sf.setBounds(142,31,651,569);
		daten.setBounds(0,0,141,600);
	
		//add components to JFrame`s content pane
		c.add(panel1);
		c.add(sf);
		c.add(daten);
	
		//add eventhandling to button and combobox
		addWindowListener(new MeinWindowAdapter());
		actionlistener al = new actionlistener();
		beenden.addActionListener(al);
		selectedplayer.addItemListener(this);
		
		//shows the window
		setVisible(true);
	}
	
	/** 
	* Displays incoming messages on the <code>JTextArea</code>. Text-flow is
	* from top to bottom, scrolling is disabled.<br/>
	*
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*
	* @param input The <code>String</code> which is to be displayed (contains Timestamp).
	* @see RoboClient#transmitTactics(String input, int source)
	* @see GUI#GUI(Player[] player)
	*/
	public void gui_Out(String input)
	{
		daten.insert("\n" + input,0);
	}
	
	/**
	* Is checking, if initialization of the Graphical User Interface worked out.
	* Sends a message to the Tactics Display, if okay.
	*
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*
	* @see RoboClient#transmitTactics(String input, int source)
	*/
	public void gui_Check()
	{
		daten.setText("\n>>>WELCOME TO RUDI \n" +">>>G37 RoboClient");
		RoboClient.transmitTactics(new String("GUI online"),0);	
	}
}












































		