/**
 * This class initializes communication, brain and worldmodel.
 * Brain needs a reference to communication and worldmodel.
 * Worldmodel nees a reference to communication
 *
 * @author <a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 */
public class Player extends Thread
{
	/** The communication of each player */
	private Communication communication;
	/** The brain of each player */
	private Brain brain;
	/** The worldmodel of each player */
	private Worldmodel world;
	/** The PlayerAction of each player */
	private PlayerAction playerAction;
		
    /**
    * Creates a new <code>Player</code> instance.
    *
    * @param servername a <code>String</code> value specifying the name of the server
    * @param serverport an <code>int</code> value specifying the port on which the server
    *                   listens for incoming packets
    */
	public Player(String servername, int serverport)
	{
		communication = new Communication(servername, serverport);
		playerAction = new PlayerAction(communication);
		world = new Worldmodel(communication);
		brain = new Brain(playerAction, this, world);
	}
	
	/** Should be called from the GUI.
	*	@return the <code>double[][]</code> positions array from the worldmodel.
	*
	*	@see Worldmodel#giveData worldmodel.giveData()*/
	public double[][] giveWorldData()
	{
		return world.giveData();
	}
	
	/** Should be called from the GUI
	*	@return the <code>boolean</code> playdirection form the worldmodel.
	*
	*	@see Worldmodel#playDirectionRight worldmodel.playDirectionRight()
	*/
	public boolean playDirectionRight()
	{
		return world.playDirectionRight();
	}
	
	/** Should be called from the GUI
	*	@return  the <code>double[]</code> body arrray form the worldmodel.
	*
	*	@see Worldmodel#bodyDoubleArray worldmodel.bodyDoubleArray()
	*/
	public double[] bodyDoubleArray()
    {
    	return world.bodyDoubleArray();
   	}
   	
	/** Should be called from the GUI
	*	@return the <code>String[]</code> hear array form the worldmodel.
	*
	*	@see Worldmodel#hearStringArray worldmodel.hearStringArray()
	*/
   	public String[] hearStringArray()
	{
		return world.hearStringArray();
	}
	
	/** Starts the brain */
	public void run()
	{
		brain.play();
	}

}