/**
 * <code>PlayerAction</code> implements the basic abilities a player can
 * perform. As an aggregation of the <code>Brain</code> class, it is supposed
 * to quickly transform tactical considerations into movement.
 *
 * @author Thorsten Fiekert
 * @author Anita Reifsteck
 * @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
 */
public class PlayerAction
{
	//atributes
	private Communication communication;
	
	//constructor	
	public PlayerAction(Communication communication)
	{
		this.communication = communication;
	}

	/** T U R N
	* turning the players body direction
	* parameter moment represents turning-angle: valid range: minmoment -180; 
	* 														   maxmoment +180;
	* @Author Fiekert */
	public void turn(double moment)
	{
		communication.send(new String("(turn " + moment + ")"));	
	}

	/** D A S H
	* dashing to the direction the player is facing
	* parameter power represents acceleration: valid range: minpower -100; 
	* 														 maxpower +100; 
	* where power < 0 means accelerating backwards [stamina costs: -2 * power]
	* @Author Fiekert */
	public void dash(double power)
	{
		communication.send(new String("(dash " + power + ")"));
	}

	/** K I C K
	* kicking the ball
	* @Author Fiekert */
	public void kick(int intensity, double direction)
    {
    	communication.send(new String("(kick " + intensity + " " + direction + ")"));
    }

	/** C A T C H
    * catching the ball
    * @Author Reifsteck */
    public void catching(double direction)
    {
    	communication.send(new String("(catch " + direction + ")"));
    }	

	/** S A Y
	* say a message
	* @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
 	*/
 	public void say(String message)
 	{
 		communication.send(new String("(say " + message + ")"));
 	}
 	
 	/** M O V E
 	* move player to a position
 	* @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	public void move(double x, double y)
	{
		communication.send(new String("(move " + x + " " + y + ")"));
	}

 	/** C H A N G E __ V I E W
 	* change_view Width Quality
 	* @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	public void change_view(String width, String quality)
	{
		communication.send(new String("(change_view " + width + " " + quality + ")"));
	}



}//end of PlayerAction