/**
 * This class contains the main method which starts 11 player-Threads 
 * and one GUI (if wanted).<br>
 * The RoboClient must be started like this:<br>
 * <code>java RoboClient [TEAMNAME [SERVERNAME [SERVERPORT]]] [-gui]</code>
 *
 * @author <a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 * @author <a href="mailto:wini.appl@epost.de">Wini Appl</a>
 */
public class RoboClient
{
	// Attribute
	/** a <code>Player</code> array with length 11 */
	private static Player[] player = new Player[11];
	
	/** a <code>GUI</code> object, which is the visualisation of the worldmodel */
	private static GUI gui;
	
	/** the teamname */
	private static String teamname = "Rudi";
	
	/** set to
    * <ul>
	* <li>  true if GUI has been started</li>
	* <li>	false if GUI turned off</li>
    * </ul>
    * as chosen in the Command Line Tool (see Documentation for further information)
    * initial value = false	
    *
    * @see RoboClient#main(String[] args)*/ 
	private static boolean startGUI;
	
	/** Stores all incoming messages in the following format :
	* 
    * <ul>
	* <li>  [1] = Timecycle message got stored in</li>
	* <li>	[2] = Actual message</li>
	* <li>	[3] = Player, which sent the message; if "0" message-priority is high</li>
    * </ul>
    *
    * @see RoboClient#transmitTactics(String input, int source)*/
	private static String[][] tactics; 
	
	/** Counter to avoid multiple denominations in the Tactics Array; points on most recent message
	* 
	*@see RoboClient#transmitTactics(String input, int source)*/
	private static int countTactics;
	
	/** Counter of the Buffer; points at last message received
	* 
	*@see RoboClient#buffer()*/
	private static int bufferIndex;
	
	/** Checks if either the Sender is a Player, or message-priority is set to high */
	private static int checkMode;
	/** set to true, if GUI took over the command line tool-output*/
	private static boolean checkGUIAlert;
		
	/** Constructor */
	public RoboClient()
	{
	
	}
	
	/** Just returns the teamname
	*
	* @return a <code>String</code> containing the teamname
	*/
	public static String tellTeamName()
	{
		return teamname;
	}

	/**
 	* This is the main method which starts 11 player-Threads 
 	* and one GUI (if wanted)
 	*
 	* @param args[] Possible arguments for the main method are:
 	* <ul>
 	* <li> teamName: Teamname
 	* <li> serverAdress: I.P. or "localhost"
 	* <li> serverPort: Port of the Server
 	* <li> <code>-gui</code>: when the visualisation (GUI) should be started
 	* </ul>
 	*
 	*  <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 	*/
	public static void main(String[] args)
	{
		/** Default value is set, initialization */
		String servername = "localhost";
		int serverport = 6000;
		startGUI = false;
		checkGUIAlert = false;
		tactics = new String[3][10000];
		for (int q=0; q<10000; q++)
		{
			tactics[1][q] = "--- no messages available yet ---";
			tactics[2][q] = "--- no data available yet ---";
		}		
		int countTactics = 0;
				
		
		// how many arguments have been given?
		switch (args.length)
		{
		// 0 arguments: start with defaults and start GUI
		case 0: startGUI = true; break;
		// 1 argument: get teamname or -gui, rest: default
		case 1: if (args[0].toLowerCase().equals("-gui")) { startGUI = true; }
				else { teamname = args[0]; }
				break;
		// 2 argument:  get teamname and servername or -gui, serverport: default
		case 2: teamname = args[0]; 
				if (args[1].toLowerCase().equals("-gui")) { startGUI = true; }
				else { servername = args[1]; }
		 		break;
		// 3 arguments: get teamname, servername and serverport or -gui
		case 3: teamname = args[0]; 
				servername = args[1];
				if (args[2].toLowerCase().equals("-gui")) { startGUI = true; }
				else {
				try {
				serverport = Integer.parseInt(args[2]);
				} catch (NumberFormatException e) // catch exception with paseInt()
				{ System.out.println("Usage: java RoboClient [TEAMNAME [SERVERNAME [SERVERPORT]]] [-gui]");
				  System.exit(1); // exit with code 1
				  break; } 
				}// end of else
				break;
		// 4 arguments: get teamname, servername and serverport and -gui
		case 4: teamname = args[0]; 
				servername = args[1];
				try {
				serverport = Integer.parseInt(args[2]);
				} catch (NumberFormatException e) // catch exception with paseInt()
				{ System.out.println("Usage: java RoboClient [TEAMNAME [SERVERNAME [SERVERPORT]]] [-gui]");
				  System.exit(1); // exit with code 1
				  break; }
				if (args[3].toLowerCase().equals("-gui")) { startGUI = true; }
				else 
				{
					System.out.println("Usage: java RoboClient [TEAMNAME [SERVERNAME [SERVERPORT]]] [-gui]");
		 		 	System.exit(1); // exit with code 1
		 		}
				break;
		// >4 arguments: exit to System
		default: System.out.println("Usage: java RoboClient [TEAMNAME [SERVERNAME [SERVERPORT]]] [-gui]");
		 		 System.exit(1); // exit with code 1
				 break;
		}
		
		// start player threads
		for (int i = 0;  i <= 10 ;i++)
		{
			player[i] = new Player(servername, serverport);
			player[i].start();
		}
		
		/** starts the GUI */
		if (startGUI == true)
		{
			gui = new GUI(player);
			gui.setVisible(true);
			gui.setSize(800,600);
			gui.gui_Check();
			flushBuffer();
		}
	
	}
	
	/** 
	* Stores all tactical computations, sent by the players and passes the
	* most actual messages on to the GUI. Buffers the remaining ones in case of
	* complications.<br/>
	*
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*
	* @param input message that is to be displayed
	* @param source sender-id of the message
	*/		
	public static void transmitTactics(String input, int source) //input should be equal or less than 16 characters
	{		
		/** 
		* Handles incoming messages as follows :
		* <ul>
		* <li> <b>1 - 11</b> Message to be displayed if according player is chosen in GUI
		* <li> <b> 0 </b> Important message from any Sender. Displayed even if sender differs from chosen player in GUI.
		* </ul>
		*/
		if (source != 0)
		{
			checkMode = source-1;
		}
		else
		{
			checkMode = source;
		}
		
		/** Timestamp is being written to tactics[0]. Before KickOff time is set to "0" */	
		try
		{
			if (player[checkMode].giveWorldData()[checkMode][5] == 1000.0) // can't use world.actualTime due to private access in player
			{
				 tactics[0][countTactics] = "0";
			}
			else
			tactics[0][countTactics] = ("" + (int)(player[checkMode].giveWorldData()[checkMode][5]) );		
		}
		/** If <code>Worldmodel</code> not yet initialized, timestamp is set  to "-" */
		catch (java.lang.NullPointerException e) 
		{
			tactics[0][countTactics] = "-";
		}
		
		/** Message is being written to tactics[1] */
		tactics[1][countTactics] = input;
		
		/** Sender of the message is written to tactics[2] */
		tactics[2][countTactics] = ("" + source);
		
		/** Formates the message and passes it on to the GUI */
		try
		{
			if (source == gui.getPlayer() || source == 0)
			{
				gui.gui_Out(new String(">" + tactics[0][countTactics] + "::" + tactics[1][countTactics]));
			}
			else
			{
			}
		}
		/** If GUI not instanciated or deactivated, the messages are buffered and stored in the tactics-array */
		catch (java.lang.NullPointerException e) 
		{
			if (startGUI == true)
			{
				if (checkGUIAlert == false)
				{
					System.out.println("<Initializing GUI> \n" + "<please wait...>");
					checkGUIAlert = true;
					buffer();
				}
				else 
				{
					buffer();
				}
			}
			else 
				if (checkGUIAlert == false)
				{
					System.out.println("<GUI deactivated>");
					checkGUIAlert = true;
				}
				else 
				{
				}			
		}
		/** Updates the Pointer */
		countTactics++;
	}
	
	/**
	* Buffers the first incoming messages for a timeinterval in which the GUI is not
	* initialized.<br/>
	*
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*/
	private static void buffer()
	{
		bufferIndex = countTactics;
	}
	
	/**
	* Writes whole buffer-content to the GUI-display, if initialized.<br/>
	*
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*/
	private static void flushBuffer()
	{
		for (int w=0; w<=bufferIndex; w++)
		{
			gui.gui_Out(new String(">" + tactics[0][w] + "::" + tactics[1][w]));
		}
	}
}