import javax.swing.*;
import java.awt.*;
import java.lang.Math;
/**
 * The <code>Spielfeld</code> class paints the playing field and the players who
 * are seen of the selected player. Also some values of see-, body- and hearevents
 * are shown.
 *
 * @author <a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
 * @author <a href="mailto:wini.appl@epost.de">Wini Appl</a>
 */


public class Spielfeld extends JPanel implements Runnable
{
	//attributes
	/**The player[] object*/
	private Player[] player;
	/**The GUI object*/
	private GUI gui;
	private Image dbImage;
    private Graphics dbGraphics;
	/**The Positionsarray*/
	private double[][] Positionen;
	/**The Positionsarray in integer*/
	private int[][] intPositionen;
	/**The player number*/
	private int myNr;

	/**
	* Constructor
	*  <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	public Spielfeld(GUI gui, Player[] player)
	{
		this.player = player;
		this.gui = gui;
		
		//two new arrays, one for the incoming player[] positionsarrey and one
		//for the integer values of this array
		Positionen = new double[23][7];
		intPositionen = new int [23][7];
	
		//initialize the integer array
		for (int a=0;a<=22;a++)
		{
			for (int b=0;b<=6;b++)
			{
				intPositionen[a][b]= 1000;
			}
		}
	}
	
	/** 
	* Transformes the array of the selected player to integer values.
	* this method is called in the thread.
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	private void getintPositionen()
	{
		Positionen = player[gui.getPlayer()-1].giveWorldData();
		for (int a=0;a<=22;a++)
		{
			for (int b=0;b<=6;b++)
			{
				intPositionen[a][b] = (int)Positionen[a][b];
			}
		}
	}		
	
	/**
	* gives the selected playernumber of the GUI back. this method is called 
	* in the thread
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
		
	private int myNr()
	{
		myNr= gui.getPlayer();
		return myNr;
	}	
	
	/**
	* Starts the thread. the method is called in the GUI. The method start() 
	* calls automatically the mothed run()in the following
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	public void startAnimation()
	{
		//new thread
		Thread th = new Thread(this);
		th.start();
	}
	
	/**
	* In the continuous loop the method repaint() is called. so all 50 ms 
	* a new playing field is painted and this give us the impression 
	* of an animation
	*  <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	public void run()
	{
		
		//continuous loop
		while (true)
		{
			
			//calls the method paint()
			repaint();
			
			getintPositionen();
			myNr();
			
			try 
			{
				
				//stopps the thread for 50 ms
         		Thread.sleep(50);
      		} 
      		
      		// all exeptions are catched
      		catch (Exception e) 
      		{
        		
        		//and printed out
        		System.err.println(e);
      		}

		}
	}
	
	
	/**
	* this method reduces the flicker of the screen by initialising 
	* a double-buffer. It is also called automatically of the repaint() method
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*/
	
	public void update(Graphics g)
    {
    	
    	//creates an image of the screen
    	if (dbImage == null) 
        {
        	dbImage = createImage(this.getSize().width,this.getSize().height);
       		dbGraphics = dbImage.getGraphics();
     	}
    	
    	//deletes the background
    	dbGraphics.setColor(getBackground());
    	dbGraphics.fillRect(0,0,this.getSize().width,this.getSize().height);
    	
   		//paints the foreground
   		dbGraphics.setColor(getForeground());
    	paint(dbGraphics);
    	
    	//shows the offscreen
    	g.drawImage(dbImage,0,0,this);
    }
    
    

	/**
	* This method paints the whole playing field. It is called every 50 ms by
	* the <code>repaint()</code> method.<br/>
	*
	* <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>,
    * <a href="mailto:wini.appl@epost.de">Wini Appl</a>
    *
    * @see Spielfeld#run()
	*/
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
		// playing field: all values are multiplied with 6. Small inaccuracies
		// because of painting is only possible with integers
		g.setColor(Color.green.darker());
		g.fillRect(0,0,650,428);
	
		// Lines
		g.setColor(Color.white);
		//middleline
		g.drawLine(325,10,325,418);
		//center circle
		g.drawOval(290,179,70,70);
		//external lines
		g.drawRect(10,10,630,408);
		//penalty
		g.drawRect(544,94,96,240);
		g.drawRect(10,94,96,240);
		//5 meters room
		g.drawRect(10,160,30,108);
		g.drawRect(610,160,30,108);
		//11 meters point
		g.drawOval(74,212,4,4);
		g.drawOval(574,212,4,4);
		//center point
		g.drawOval(323,212,4,4);
		//goal
		g.setColor(Color.black);
		g.fillRect(640,172,5,84);
		g.fillRect(5,172,5,84);
		
		// Shows the x-position of the selected player
		g.drawString("X-Pos.: "+ intPositionen[myNr][0],5,460);
		// Shows the y-position of the selected player
		g.drawString("Y-Pos.: "+ intPositionen[myNr][1],120,460);
		// Shows the speed-direction of the selected player
		g.drawString("Speed Dir.: "+(int)player[myNr-1].bodyDoubleArray()[6],350,460);
		// Shows the timecyclus
		g.drawString("Timecyclus: "+intPositionen[myNr][5],465,460);
		
		// Shows the view-mode of the selected player
		if (player[myNr-1].bodyDoubleArray()[2] == 0)
		g.drawString("view mode:  narrow",235,460);
		if (player[myNr-1].bodyDoubleArray()[2] == 1)
		g.drawString("view mode:  normal",235,460);
		if (player[myNr-1].bodyDoubleArray()[2] == 2)
		g.drawString("view mode:  wide",235,460);
		
		// Shows the stamina of the selected player
		g.drawString("Stamina: "+(int)player[myNr-1].bodyDoubleArray()[3],5,480);
		// Shows the head-angle of the selected player
		g.drawString("Head-Angle: "+(int)player[myNr-1].bodyDoubleArray()[7],120,480);
		// Shows the dash-count of the selected player
		g.drawString("Dash-Count: "+(int)player[myNr-1].bodyDoubleArray()[9],235,480);
		// Shows the kick-count of the selected player
		g.drawString("Kick-Count: "+(int)player[myNr-1].bodyDoubleArray()[8],350,480);
		// Shows the turn-count of the selected player
		g.drawString("Turn-Count: "+(int)player[myNr-1].bodyDoubleArray()[10],465,480);
		
		// shows some values of the hear messages of the selected player
		g.drawString("Hear-Events:",5,500);
		// Shows the sender
		g.drawString("Sender: "+player[myNr-1].hearStringArray()[1],235,500);
		// Shows the message
		g.drawString("Message: "+player[myNr-1].hearStringArray()[2],5,520);
		
		/** If play direction is from left to right */
		if (player[myNr-1].playDirectionRight())
		{
			/** Length of line which represents the player's line of sight */
			int dist = 10;
			/** Player's momentary position along x-axis*/
			int nowX = (6*intPositionen[myNr][0])+325;
			/** Player's momentary position along y-axis*/
			int nowY = (6*intPositionen[myNr][1])+214;
			/** Player's momentary viewing-vector along x-axis*/
			int lookX = (int)((Math.cos(intPositionen[myNr][6]* Math.PI / 180)) * dist );
			/** Player's momentary viewing-vector along y-axis*/
			int lookY = (int)((Math.sin(intPositionen[myNr][6]* Math.PI / 180)) * dist );
			
			/** Paints the selected player in yellow */
			g.setColor(Color.yellow);
			g.fillOval(nowX - 5,nowY - 5,10,10);
			/** Writes the selected player's number below the Monitor */			
			g.drawString("Player "+myNr+" is yellow",50,440);
			
			g.setColor(Color.blue.darker());
			/** Draws Player's line of sight*/
			g.drawLine(nowX, nowY, nowX+lookX, nowY+lookY);
			/** Writes Player's number*/
			g.drawString("" + myNr, nowX +5, nowY -5);
			
			/** Length of line which represents the player's line of sight */
			int distFriend = 5;
			
			/** Computes momentary position of each of our fellow players */
			for (int i=1;i<=11;i++)
			{
				if(Positionen[myNr][5]== Positionen[i][5] && i!= myNr)
				{
					/** Player's momentary position along x-axis*/
					int nowXFriend = (6*intPositionen[i][0])+325;
					/** Player's momentary position along y-axis*/
					int nowYFriend = (6*intPositionen[i][1])+214;
					/** Player's momentary viewing-vector along x-axis*/
					int lookXFriend = (int)((Math.cos(intPositionen[i][6]* Math.PI / 180)) * distFriend );
					/** Player's momentary viewing-vector along y-axis*/
					int lookYFriend = (int)((Math.sin(intPositionen[i][6]* Math.PI / 180)) * distFriend );
					
					/** Paints team mates blue */
					g.setColor(Color.blue);
					g.fillOval(nowXFriend - 5, nowYFriend - 5, 10, 10);
					
					g.setColor(Color.yellow);
					/** Draws Player's line of sight*/
					g.drawLine(nowXFriend, nowYFriend, nowXFriend + lookXFriend, nowYFriend + lookYFriend);
					/** Writes Player's number*/
					g.drawString("" + i, nowXFriend +5, nowYFriend -5);
				}
			}
			/** Writes color-code below the Monitor */
			g.setColor(Color.blue);
			g.drawString("Teammates are blue",260,440);
			
			
			// opponent
			g.setColor(Color.red);
			for (int n=12;n<=22;n++)
			{
				if(Positionen[myNr][5]== Positionen[n][5])
				{
					g.fillOval((6*intPositionen[n][0])+320,(6*intPositionen[n][1])+209,10,10);
					g.drawString("" + n,(6*intPositionen[n][0])+320,(6*intPositionen[n][1])+209);
				}
			}
			g.drawString("Opponents are red",480,440);
			
			// ball
			g.setColor(Color.white);
			if (Positionen[myNr][5]== Positionen[0][5])
			{
				g.fillOval((6*intPositionen[0][0])+322,(6*intPositionen[0][1])+221,6,6);
			}
		}	
		
		/** Ruuudii is playing from right to left */
		else   
		{	
		
			/** Length of line which represents the player's line of sight */
			int dist = 10;
			/** Player's momentary position along x-axis*/
			int nowX = 325 -(6*intPositionen[myNr][0]);
			/** Player's momentary position along y-axis*/
			int nowY = 214 - (6*intPositionen[myNr][1]);
			/** Player's momentary viewing-vector along x-axis*/
			int lookX = (int)((Math.cos(intPositionen[myNr][6]* Math.PI / 180)) * dist );
			/** Player's momentary viewing-vector along y-axis*/
			int lookY = (int)((Math.sin(intPositionen[myNr][6]* Math.PI / 180)) * dist );
			
			/** Paints the selected player in yellow */
			g.setColor(Color.yellow);
			g.fillOval(nowX - 5,nowY - 5,10,10);
			/** Writes the selected player's number below the Monitor */			
			g.drawString("Player "+myNr+" is yellow",50,440);
			
			g.setColor(Color.blue.darker());
			/** Draws Player's line of sight*/
			g.drawLine(nowX, nowY, nowX-lookX, nowY-lookY);
			/** Writes Player's number*/
			g.drawString("" + myNr, nowX +5, nowY -5);
			
			
			/** Length of line which represents the player's line of sight */
			int distFriend = 5;
			
			/** Computes momentary position of each of our fellow players */
			for (int i=1;i<=11;i++)
			{
				if(Positionen[myNr][5]== Positionen[i][5] && i!= myNr)
				{
					/** Player's momentary position along x-axis*/
					int nowXFriend = 325 - (6*intPositionen[i][0]);
					/** Player's momentary position along y-axis*/
					int nowYFriend = 214 - (6*intPositionen[i][1]);
					/** Player's momentary viewing-vector along x-axis*/
					int lookXFriend = (int)((Math.cos(intPositionen[i][6]* Math.PI / 180)) * distFriend );
					/** Player's momentary viewing-vector along y-axis*/
					int lookYFriend = (int)((Math.sin(intPositionen[i][6]* Math.PI / 180)) * distFriend );
					
					/** Paints team mates blue */
					g.setColor(Color.blue);
					g.fillOval(nowXFriend - 5, nowYFriend - 5, 10, 10);
					
					g.setColor(Color.yellow);
					/** Draws Player's line of sight*/
					g.drawLine(nowXFriend, nowYFriend, nowXFriend - lookXFriend, nowYFriend - lookYFriend);
					/** Writes Player's number*/
					g.drawString("" + i, nowXFriend +5, nowYFriend -5);
				}
			}
			/** Writes color-code below the Monitor */
			g.setColor(Color.blue);
			g.drawString("Teammates are blue",260,440);
			
			// opponent
			g.setColor(Color.red);
			for (int n=12;n<=22;n++)
			{
				if(Positionen[myNr][5]== Positionen[n][5])
				{
					g.fillOval(320-(6*intPositionen[n][0]),209-(6*intPositionen[n][1]),10,10);
					g.drawString("" + n,320-(6*intPositionen[n][0]),209-(6*intPositionen[n][1]));
				}
			}
			g.drawString("Opponents are red",480,440);
			
			// ball
			g.setColor(Color.white);
			if (Positionen[myNr][5]== Positionen[0][5])
			{
				g.fillOval(322-(6*intPositionen[0][0]),221-(6*intPositionen[0][1]),6,6);
			}		
		}	
	}	
}
		
		
	
	
	
	
	

