import java.lang.Math;
import java.util.*;
/**
 * The <code>Worldmodel</code> class is used to calculate absolute positions of 
 * seen objects with their given relative data. Most important is the calculation
 * of the own position. For that a field line, which normaly can be seen, and the
 * nearest seen flag are used; Further body- and hearevents recieved from
 * Parser class are delivered to Brain class. 
 *
 * @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
 * @author <a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
 * @author <a href="wini.appl@epost.de">Wini Appl</a>
 * @author <a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 * @author <a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
 * @author <a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
 */
public class Worldmodel
{
		/** The Communication object */
		private Communication communication;
		/** The Parser object */
		private Parser parser;
		/** The Vector object */
		private Vector vector;
	
			
			/** positions = array[][] with double[] of length 7 with following content:
	        * <ul>
			* <li>  first positions [x][]:
			* <li>	[0]			: ball
			* <li>	[1] - [11]	: player of own team related to player-number
			* <li>	[12]- [22]	: player of the other team related to player-number + 11
			* <li>  second positions [][x]:
			* <li>	[0]			: x-position
			* <li>	[1]			: y-position
			* <li>	[2]			: by rccs-server given distance
			* <li>	[3]			: by rccs-server given angle (no absolute angle)
			* <li>	[4]			: by rccs-server given distanceChange - not used
			* <li>	[5]			: timecyclus
			* <li>	[6]			: absolute angle (in case of ownNumber) and distance to ball (in case of other player)
            * </ul>
            * initialazion value is 1000.0	
			* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	        */ 	
		private double[][] positions;
		
		private boolean playerChanged;
		private boolean playDirectionRight; //play-direction --->
		private int myNr;
		private String teamName;
	
			
	 		/** hearStringArray = String array with following content:
			*  <ul>
	 		*  <li> [0]: time
	 		*  <li> [1]: sender { "referee" | "self" | direction | "online_coach_left" | "online_coach_right"}
	 		*  <li> [2]: message
	 		*  </ul>
	 		*  time can be converted to int
	 		*  direction can be converted to double
	 		*  <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	 		*/
		private String[] hearStringArray;
		private String refereeMsg;
		
		
	 	      /** bodyDoubleArray = array with length 16 and:
			* <ul>
	 		* <li>	[0]: time
	 		* <li>	[1]: view_mode(1){ 0="high" | 1="low" }
	 		* <li>	[2]: view_mode(2){ 0="narrow" | 1="normal" | 2="wide" }
	 		* <li>	[3]: stamina(1)	( Stammina )
	 		* <li>	[4]: stamina(2)	( Effort )
	 		* <li>	[5]: speed(1)	( AmountOfSpeed )	
	 		* <li>	[6]: speed(2)	( DirectionOfSpeed )[-180 ~ 180 degrees]
	 		* <li>	[7]: head-angle	( HeadAngle )		[-180 ~ 180 degrees]
	 		* <li>	[8]: kick		( KickCount ) 		[positive integer]
	 		* <li>	[9]: dash		( DashCount ) 		[positive integer]
	 		* <li>	[10]: turn		( TurnCount ) 		[positive integer]
	 		* <li>	[11]: say		( SayCount ) 		[positive integer]
	 		* <li>	[12]: turn_neck	( TurnNeckCount )	[positive integer]
	 		* <li>	[13]: catch		( CatchCount )		[positive integer]
	 		* <li>	[14]: move		( MoveCount )		[positive integer]
	 		* <li>	[15]: change_view ( ChangeViewCount )[positive integer]
	 		* </ul>
	 		* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	 	     **/ 
		private double[] bodyDoubleArray;
		
		private boolean initialized;
    	private int actualTime;
    	
    	/** event = array with following content:
		*  <ul>
	 	*  <li> [0]: time
	 	*  <li> [1]: { 0=hear | 1=visual | 2=body }
	 	*/
    	private int[] event; 
    	private int[] parsedID;
    		
	/**		
	* Creates a new <code>Worldmodel</code> instance.
	* @param communication a <code>Communication</code> value defining the
    * communication link to the server via which the commands can be
    * sent.
    * @see Communication
	*/
	public Worldmodel(Communication communication)
		{
			this.communication = communication;
			parser = new Parser(communication);
			vector = new Vector();
			positions = new double[23][7];
			playDirectionRight = false;
			teamName = RoboClient.tellTeamName();
			myNr = 0;
			refereeMsg = "no Message";
			hearStringArray = new String[3];
			bodyDoubleArray = new double[16];
			initialized = false;
			playerChanged = false;
			actualTime = 0;
			event = new int[2];	
			parsedID = new int[2];
		}

	/**
	*  @return the <code>String</code> referee message
	*/
	public String playmode()
	{
		return refereeMsg;
	}
	
	/**
	*  @return the <code>String[]</code> hear array
	*/
	public String[] hearStringArray()
		{
			return hearStringArray;
		}
		
	/**
	*  @return the <code>integer</code> actual Time
	*/
	public int actualTime()
		{
			return actualTime;
		}

	/**
	*  @return the <code>boolean</code> attribute initialized
	*/
	public boolean initialized()
		{ 
			return initialized;
		}
		
	/**
	*  @return the <code>integer</code> Player number
	*/
	public int playerNr()
		{
			return myNr;
		}
	
	
	/**
	*   @return the <code>double[][]</code> positions array
	*/
	public double[][] giveData()
		{
			return positions;			
		}

	/**
	*   @return the <code>int[]</code> event array
	*/
	public int[] event()
		{
			return event;
		}	
    
    /**
    *   @return the <code>double[]</code> body arrray
    */
    public double[] bodyDoubleArray()
    	{
    	   return bodyDoubleArray;
    	}
    	
    
    /** Should be called from the GUI
    *   @return the <code>boolean</code> playdirection
    */
    public boolean playDirectionRight()
    {
    	return playDirectionRight;
    }    

	/**
	* The <code>refereeMsg</code> method receive and set referee messages 
	*
	* <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	private void refereeMsg()
	{
		if( hearStringArray()[1].equals("referee") )
			refereeMsg = hearStringArray()[2];
	}

	/** 
	*  The <code>init</code> method initializates players and gets information 
	*  from server about playdirection, own Number and playmode
	*
	*  <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
	*/
	public void init()
		{
			// connect as a goalie
			communication.send(new String("(init " + teamName + " (version 7.0) (goalie))"));
		
			String side = "";
			String msg = "";
			String playMode = "";
		
    	    //while ((msg.startsWith("(init")==false) | (msg.startsWith("(error")==false))
    	    while (msg.equals(""))
    	    {
				// getting init-message
				msg = communication.receive();
    		}
    		//if player is first player he is the goalie
    		if (msg.startsWith("(init"))
   			{
    				// i.e. for an init-message: (init l 1 playmode)
    				msg = msg.substring(6, msg.length()-1);
    				StringTokenizer st = new StringTokenizer(msg);
    			
    				// setting of playdirection
    				side = st.nextToken();
    				if (side.equals("l")) playDirectionRight = true; // --->
    					else playDirectionRight = false; // <---
    			
    				// setting of own Number
    				myNr = Integer.parseInt(st.nextToken());
    			
    				// setting of playmode (goalie - yes or no)
    				if (st.hasMoreTokens()) playMode = st.nextToken();
   		 		}
   		 	//if player is not the first player an error msg is returned
   		 	else if (msg.startsWith("(error no_more_team_or_player_or_goalie"))
   		 		 {
   		 		 	//connect as a field player
   		 		 	communication.send(new String("(init " + teamName + " (version 7.0))"));
   		 		 	while ((msg.equals("")) | (msg.startsWith("(init")==false))
    	    		{
						// getting init-message
						msg = communication.receive();
    				}
    				if (msg.startsWith("(init"))
    				{
    				// i.e. for an init-message: (init l 1 playmode)
    				msg = msg.substring(6, msg.length()-1);
    				StringTokenizer st = new StringTokenizer(msg);
    			
    				// setting of playdirection
    				side = st.nextToken();
    				if (side.equals("l")) playDirectionRight = true; // --->
    					else playDirectionRight = false; // <---
    			
    				// setting of own Number
    				myNr = Integer.parseInt(st.nextToken());
    			
    				// setting of playmode (goalie - yes or no)
    				if (st.hasMoreTokens()) playMode = st.nextToken();
   		 			}
   		 		}

			//initializiations
			initialized=true;
			hearStringArray[0] = "";
			hearStringArray[1] = "";
			hearStringArray[2] = "";
			playerChanged = false;
			
			for (int r = 0; r <= 22;r++)
			{
				positions[r][0] = 1000.0; //all beginnig values of positions[][] are 1000
				positions[r][1] = 1000.0;
				positions[r][2] = 1000.0;
				positions[r][3] = 1000.0;
				positions[r][4] = 1000.0;
				positions[r][5] = 0.0;
				positions[r][6] = 1000.0;
			}
			
			//these are the start positions of all players of our team
			
			positions[0][0] =   0;	positions[0][1] =   0;
			positions[1][0] = -50; 	positions[1][1] =   0;
			positions[2][0] = -35; 	positions[2][1] = -20;
			positions[3][0] = -35; 	positions[3][1] =   0;
			positions[4][0] = -35; 	positions[4][1] =  20;
			positions[5][0] = -20;	positions[5][1] = -25;
			positions[6][0] = -20; 	positions[6][1] =  -8;
			positions[7][0] = -20; 	positions[7][1] =   8;
			positions[8][0] = -20; 	positions[8][1] =  25;
			positions[9][0] =  -5; 	positions[9][1] = -20;
			positions[10][0] = -5; 	positions[10][1] =  0;
			positions[11][0] = -5; 	positions[11][1] = 20;
			
			/* 
			* sends an okay-message to the tactics-array if everything has worked out well
			*
			* @see RoboClient#transmitTactics 
			* @author <a href="wini.appl@epost.de">Wini Appl</a>
			*/
			RoboClient.transmitTactics(new String("Player "+myNr+" online"),0);
			
	} // end of private void init()
	
	
	public void disconnect()
	{
		/** author <a href="wini.appl@epost.de">Wini Appl</a> */
		RoboClient.transmitTactics(new String("Connection closed"),0);	
		
		communication.send("(bye)");
		System.exit(0);
	}	
		
		
		

    /** The <code>flagPosition</code> method gives absolute flagposition back
    *
    *  @param flagName a <code>VisualObject</code> objectName
    *  @return the <code>double[]</code> absolute flagposition array
	*  <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*  <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	*  <P><b>Author: </b><a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
	*/
	private double[] flagPosition(VisualObject flagName)
	{
		String flagPos = flagName.objectName();
		double x = 100;
		double y = 100;
		double[] position = new double[2];
	
		int counter = 0;
			
		while (x == 100) 
		{
			switch (counter)
			{
				case 0:	if (flagPos.equals("fc"))  	{x=0.0;y=0.0;		}	break;
				case 1: if (flagPos.equals("fct")) 	{x=0.0;y=-34.0;		}	break;
				case 2: if (flagPos.equals("fcb")) 	{x=0.0;y=34.0;		}	break;
				case 3: if (flagPos.equals("fprt"))	{x=36.5;y=-20.0;	}	break;
    			case 4: if (flagPos.equals("fprc"))	{x=36.5;y=0.0;		}	break;
				case 5: if (flagPos.equals("fprb"))	{x=36.5;y=20.0;		}	break;
				case 6: if (flagPos.equals("frt")) 	{x=52.5;y=-34.0;	}	break;
				case 7: if (flagPos.equals("fgrt"))	{x=52.5;y=-7.01;	}	break;
				case 8: if (flagPos.equals("gr"))  	{x=52.5;y=0;		}	break;
				case 9: if (flagPos.equals("fgrb"))	{x=52.5;y=7.01;		}	break;
				case 10:if (flagPos.equals("frb")) 	{x=52.5;y=34;		}	break;
				case 11:if (flagPos.equals("fplt"))	{x=-36.5;y=-20;		}	break;
				case 12:if (flagPos.equals("fplc"))	{x=-36.5;y=0;		}	break;
				case 13:if (flagPos.equals("fplb"))	{x=-36.5;y=20;		}	break;
				case 14:if (flagPos.equals("flt")) 	{x=-52.5;y=-34;		}	break;
				case 15:if (flagPos.equals("fglt"))	{x=-52.5;y=-7.01;	}	break;
				case 16:if (flagPos.equals("gl"))	{x=-52.5;y=0;		}	break;
				case 17:if (flagPos.equals("fglb"))	{x=-52.5;y=7.01;	}	break;
				case 18:if (flagPos.equals("flb"))	{x=-52.5;y=34;		}	break;
				case 19:if (flagPos.equals("ftl50")){x=-50;y=-39;		}	break;
				case 20:if (flagPos.equals("ftl40")){x=-40;y=-39;		}	break;
				case 21:if (flagPos.equals("ftl30")){x=-30;y=-39;		}	break;
				case 22:if (flagPos.equals("ftl20")){x=-20;y=-39;		}	break;
				case 23:if (flagPos.equals("ftl10")){x=-10;y=-39;		}	break;
				case 24:if (flagPos.equals("ft0"))	{x=0;y=-39;			}	break;
				case 25:if (flagPos.equals("ftr10")){x=10;y=-39;		}	break;
				case 26:if (flagPos.equals("ftr20")){x=20;y=-39;		}	break;
				case 27:if (flagPos.equals("ftr30")){x=30;y=-39;		}	break;
				case 28:if (flagPos.equals("ftr40")){x=40;y=-39;		}	break;
				case 29:if (flagPos.equals("ftr50")){x=50;y=-39;		}	break;
				case 30:if (flagPos.equals("frt30")){x=57.5;y=-30;		}	break;
				case 31:if (flagPos.equals("frt20")){x=57.5;y=-20;		}	break;
				case 32:if (flagPos.equals("frt10")){x=57.5;y=-10;		}	break;
				case 33:if (flagPos.equals("fr0"))	{x=57.5;y=0;		}	break;
				case 34:if (flagPos.equals("frb10")){x=57.5;y=10;		}	break;
				case 35:if (flagPos.equals("frb20")){x=57.5;y=20;		}	break;
				case 36:if (flagPos.equals("frb30")){x=57.5;y=30;		}	break;
				case 37:if (flagPos.equals("fbr50")){x=50;y=39;			}	break;
				case 38:if (flagPos.equals("fbr40")){x=40;y=39;			}	break;
				case 39:if (flagPos.equals("fbr30")){x=30;y=39;			}	break;
				case 40:if (flagPos.equals("fbr20")){x=20;y=39;			}	break;
				case 41:if (flagPos.equals("fbr10")){x=10;y=39;			}	break;
				case 42:if (flagPos.equals("fb0"))	{x=0;y=39;			}	break;
				case 43:if (flagPos.equals("fbl10")){x=-10;y=39;		}	break;
				case 44:if (flagPos.equals("fbl20")){x=-20;y=39;		}	break;
				case 45:if (flagPos.equals("fbl30")){x=-30;y=39;		}	break;
				case 46:if (flagPos.equals("fbl40")){x=-40;y=39;		}	break;
				case 47:if (flagPos.equals("fbl50")){x=-50;y=39;		}	break;
				case 48:if (flagPos.equals("flb30")){x=-57.5;y=30;		}	break;
				case 49:if (flagPos.equals("flb20")){x=-57.5;y=20;		}	break;
				case 50:if (flagPos.equals("flb10")){x=-57.5;y=10;		}	break;
				case 51:if (flagPos.equals("fl0"))	{x=-57.5;y=0;		}	break;
				case 52:if (flagPos.equals("flt10")){x=-57.5;y=-10;		}	break;
				case 53:if (flagPos.equals("flt20")){x=-57.5;y=-20;		}	break;
				case 54:if (flagPos.equals("flt30")){x=-57.5;y=-30;		}	break;
			}
		
			if (counter == 55)	{	x=0.0;y=0.0;	}
			counter++;
		}
	
	   	if (playDirectionRight)	// --->
    	{
    		position[0] = x;
    		position[1] = y;
    	}
    		else					// <---
    		{
    			position[0] = x * (-1.0);
    			position[1] = y * (-1.0);
    		}
    				
//--------- test: benutzte Flag wird im frhsten Erkennstadium ausgegeben
//	System.out.println ("Spieler:"+myNr+" flagPosX:"+position[0]+" flagPosY:"+position[1]);
//---------
	
		return position;	
	}


    /** The <code>getNextFlag</code> method takes the nearest flag of all seen ones 
     * 
     * @return a double array with following content:
	 * <ul>
	 * <li> [0]	: absolute X-position of the flag
     * <li> [1]	: absolute Y-position of the flags
     * <li> [2]	: distance to the flag (passed by server)
     * <li> [3]	: angle to the flag (passed by server)
     * </ul>
     *
	 * <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	 * <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
     */
    private double[] getNextFlag()
    {	
    	int bestObjNr=-1; // number of nearest flag
    	double dist = 1000.0; // initialization of unreal value
    	VisualObject vo;
    	
    		// search for nearest flag
    		for (int i = 0; i < vector.size(); i++)
			{
				vo = (VisualObject)vector.get(i);
    			
    			if (vo.objectName().startsWith("f"))
    			{
    				if ( vo.distance() < dist ) 
    				{
    					bestObjNr = i;
    					dist = vo.distance();
    				}
    			}
    		}
    	
    	double[] nextFlag = new double[4];
    	
    	if (bestObjNr > -1)
    	{
    		vo = (VisualObject)vector.get(bestObjNr);
    		double[] getFlag = new double[2];
    		getFlag = flagPosition(vo);
    	    	    	
    		nextFlag[0] = getFlag[0];
    		nextFlag[1] = getFlag[1];
    		nextFlag[2] = vo.distance();
    		nextFlag[3] = vo.angle();
    	}
    		else // in case that no flag is seen
    		{
    			nextFlag[0] = 1000.0;
    			nextFlag[1] = 1000.0;
    			nextFlag[2] = 1000.0;
    			nextFlag[3] = 1000.0;
    		}
  
//-------------- test: verwendete Flag wird ausgegeben
//	System.out.println ("Spieler:"+myNr+" FlagX:"+nextFlag[0]+" FlagY:"+nextFlag[1]+" disFlag:"+nextFlag[2]+" Winkel:"+nextFlag[3]);
//--------------  	
    	
   		return nextFlag;		
    } // end of private double[] getNextFlag()	

    
    
     /** The <code>getLineAngle</code> method gives seen line with see-angle back
     * 
     * @return a double array with following content:
	 * <ul>
	 * <li> [0]	: { 0.0=top, 1.0=right, 2.0=bottom, 3.0=left }
     * <li> [1]	: angle 
     * </ul>
     *
     *  <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	 *  <P><b>Author: </b><a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
	 *  <P><b>Author: </b><a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
	 */
    private double[] getLineAngle()
    {
    	double[] line = new double[2];
    	VisualObject LineName;
    	line[0] = -1.0;
    		
    	int i = vector.size(); i--; // vector shall been searched from last to first position
    	while (i > 0)
    	{
    		LineName = (VisualObject)vector.get(i);
    		if (LineName.objectName().equals("lt"))
    		{
    			line[0] = 0.0;
    			line[1] = LineName.angle();
    			i = -1;
    		}
    		if (LineName.objectName().equals("lr"))
    		{
    			line[0] = 1.0;
    			line[1] = LineName.angle();
    			i = -1;
    		}
    		if (LineName.objectName().equals("lb"))
    		{
    			line[0] = 2.0;
    			line[1] = LineName.angle();
    			i = -1;
    		}
    		if (LineName.objectName().equals("ll"))
    		{
    			line[0] = 3.0;
    			line[1] = LineName.angle();
    			i = -1;
    		}	
    		i--; // vector shall been searched from last to first position
	   	}
	   	
	   	if (line[0] < 0)
	   	{
	   		line[0] =1000; // unreal value
	   		line[1] =1000; // unreal value
	   	}

//------------- test: verwendete Linie wird ausgegeben
//	System.out.println ("Spieler:"+myNr+" Line:"+line[0]+" LineAngle:"+line[1]);
//------------- 			
    		
   		return line;
	}
    
    
    /** The <code>rel2abs</code> method calculates absolute values with by 
     *  server passed information.
     *
     * @param distance a <code>double</code> 
     * @param angle a <code>double</code> 
     * @return a double array with following content:
	 * <ul>
	 * <li> [0]	: x-value
     * <li> [1]	: y-value
     * </ul>
     *
     *  <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
    */
    private double[] rel2abs(double distance, double angle)							
    {
   	   double[] absolut = new double[2];
    	   
   	   absolut[0] = positions[myNr][0] + distance * Math.cos((positions[myNr][6]+angle)*Math.PI/180);
   	   absolut[1] = positions[myNr][1] + distance * Math.sin((positions[myNr][6]+angle)*Math.PI/180);


   	   return absolut;
    }
        
    
    /** The <code>calcOwnPos</code> method calculates ownPosition and uses getNextFlag and getLineAngle
    *   fills positions[myNr][0,1,5,6]
    *
    *   <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
    */
    private void calcOwnPos()
    {
   		double[] nextFlag = getNextFlag();
   		double[] line = getLineAngle();
		
		if (nextFlag[0] < 1000 && line[0] < 1000) //checks if a flag and a line is seen
		{
	   		double flag_x = nextFlag[0];
   			double flag_y = nextFlag[1];
   			double flagdis = nextFlag[2];
   			double flagangle = nextFlag[3];
    		
    	
   		 	//linetyp : 0.0=top, 1.0=right, 2.0=bottom, 3.0=left
   			double linetyp = line[0];
   			double lineangle = line[1];
    		
    		double absangle = 1000.0; //unreal value
    		   		
    		if (linetyp == 0.0) //top
    		{
    			if (playDirectionRight)
    				{
    					if (lineangle < 0.0)					// --->
    					absangle = -180.0 - lineangle;
    					else absangle = 0.0 - lineangle;
    				}	
    				else									// <---
    				{	if (lineangle > 0.0)
    						absangle = 180.0 - lineangle;
    						else absangle = 0.0 - lineangle;
    				}
    		}
    		
    		if (linetyp == 1.0) //right
    		{
    			if (playDirectionRight)					// --->
    			{
    				if (lineangle < 0.0)
    					absangle = -90.0 - lineangle;
    					else absangle = 90.0 - lineangle;
    			}
    				else									// <---
    				{	if (lineangle > 0.0)
    						absangle = -90.0 - lineangle;
    						else absangle = 90.0 - lineangle;
    				}
    		}
    		
    		if (linetyp == 2.0) //bottom
    		{
    			if (playDirectionRight)					// --->
    				{
    				if (lineangle < 0.0)
    					absangle = 0.0 - lineangle;
    					else absangle = 180.0 - lineangle;
    				}	
    				else									// <---
    				{	if (lineangle < 0.0)
    						absangle = -180.0 - lineangle;
    						else absangle = 0.0 - lineangle;
    				}
    		}
    		
    		if (linetyp == 3.0) //left
    		{
    			if (playDirectionRight)					// --->
    			{
    				if (lineangle > 0.0)
    					absangle = -90.0 - lineangle;
    					else absangle = 90.0 - lineangle;
    			}
    				else									// <---
    				{	if (lineangle < 0.0)
    						absangle = -90.0 - lineangle;
    						else absangle = 90.0 - lineangle;
    				}
    				    					
    		}
    	    	    		
    		positions[myNr][0] = flag_x - (flagdis * Math.cos((absangle + flagangle) * Math.PI / 180));
    		positions[myNr][1] = flag_y - (flagdis * Math.sin((absangle + flagangle) * Math.PI / 180));
    		positions[myNr][5] = actualTime;
    		positions[myNr][6] = absangle;
    	}
    		else // if no line or no flag can be seen
    		{
    			positions[myNr][0] = 1000; // unreal value
    			positions[myNr][1] = 1000; // unreal value
    			positions[myNr][5] = actualTime;
    		}
    		
    	
//-------------- test: Benutzte Flag zur Positionsbestimmung
//	System.out.println("Spieler:"+myNr+" time"+actualTime+" flagX:"+flag_x+" flagY:"+flag_y+" flagDis:"+flagdis+" flagAngle:"+flagangle);	
//--------------  	

//-------------- test: Eigene Position
//	System.out.println("Sp"+myNr+" time"+actualTime+" PosX:"+positions[myNr][0]+" PosY:"+positions[myNr][1]+" Angle:"+positions[myNr][2]);
//--------------
	}

	
	/**
	*  The <code>calcPositions</code> method calculates absolute positions of seen objects.
	*  uses methods calcOwnPos() and rel2abs()
	*
	*   <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
    */
	private void calcPositions()
	{
			calcOwnPos();
			if (positions[myNr][0] < 1000) //checks, wether own position could been calculated
			{
				VisualObject vo;
				int objNum = 0;
				
				for (int i = 0; i < vector.size(); i++)
				{
					vo = (VisualObject)vector.get(i);
					if (vo.objectName().startsWith("b"))
					{
						double[] abs = rel2abs((double)vo.distance(),(double)vo.angle());
				
						positions[0][0] = abs[0];					//X-position
						positions[0][1] = abs[1];					//Y-position
					
						positions[0][2]	= vo.distance();			//by server passed distance 
						positions[0][3] = vo.angle();				//by server passed angle
						positions[0][4]	= vo.distChange();			//by server passed distanceChange
					
						positions[0][5] = actualTime*1.0;			//timecyclus

//-------------- test: Position des Balls
//	System.out.println ("Sp"+myNr+" time"+actualTime+" BallX:"+positions[0][0]+" BallY:"+positions[0][1]);
//--------------
				
					}
				
					if (vo.objectName().startsWith("p"))
					{
						// only seen playernumbers are relavant
						if (vo.uniformNr() > 0)
						{
							if (vo.teamName().equals(teamName))
								objNum = vo.uniformNr();
								else objNum = vo.uniformNr() + 11;
					
							double[] abs = rel2abs((double)vo.distance(),(double)vo.angle());
				
							positions[objNum][0] = abs[0];				//X-position
							positions[objNum][1] = abs[1];				//Y-position
					
							positions[objNum][2] = vo.distance();		//by server passed distance
							positions[objNum][3] = vo.angle();			//by server passed angle
							positions[objNum][4] = vo.distChange();		//by server passed distanceChange
					
							positions[objNum][5] = actualTime*1.0;		//timecyclus


//-------------- Testausgabe: bergebene Werte an rel2abs
//	System.out.println("Sp"+myNr+" t_s"+positions[objNum][5]+" ArrayPos"+vo.uniformNr()+" dis"+vo.distance()+" angle"+vo.angle());
//--------------

//-------------- Testausgabe: Position eines gesichteten Spielers
//	System.out.println ("Sp"+myNr+" t_s"+positions[objNum][5]+" Team: "+vo.teamName()+" Triko"+vo.uniformNr()+" Pl_X:"+positions[objNum][0]+" Pl_Y:"+positions[objNum][1]);
//--------------
						}
					}
				
				}
				// calculation of distance between players and the ball (positions[x][6])
				for (int ramin =1; ramin < 23 ;ramin++)
				{
					if (ramin != myNr && positions[ramin][5] == actualTime)
					{
						positions[ramin][6] = Math.sqrt(Math.pow(positions[ramin][0]-positions[0][0],2)+Math.pow(positions[ramin][1]-positions[0][1],2));
					}
				}

//-------------- Testausgabe: komplettes Positions - Array
/*	System.out.println("Sp"+myNr+" t_a"+actualTime+" PosX:"+positions[myNr][0]+" PosY:"+positions[myNr][1]+" Angle:"+positions[myNr][6]);
	for (int r = 1; r < 23; r++)
	{
		System.out.println ("Sp"+myNr+" t_a"+actualTime+" ArrayPos"+r+" t_s"+positions[r][5]+" Pl_X:"+positions[r][0]+" Pl_Y:"+positions[r][1]+" dis"+positions[r][2]+" ang"+positions[r][3]);
	}
	System.out.println ("Sp"+myNr+" t_a"+actualTime+" t_s"+positions[0][5]+" BallX:"+positions[0][0]+" BallY:"+positions[0][1]);
	/*
	*/
//--------------
			} // end of - if (positions[myNr][0] < 1000)
	} // end of public void calcPositions()

	/**	 The <code>run</code> method differes between hear, see and body events
	*
	*	<P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*   <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
	*/
	public void run()
	{
			event[0]=0; event[1]=1;
			// call parser and wait for returned vector
			vector = parser.parse();
			parsedID[1]=3;
//------------			
//	System.out.println("Spieler:"+myNr+"ELEMENTE IM VECTOR: " + vector.size());
//------------

			/** get content of vector (HearEvent=0, VisualEvent=1 or BodyEvent=2)
			* saved in parsedID[1]
			* parsedID[0] contains the timecyclus of the event
			*/
			if (vector.size()>0)
			{ 
				parsedID = (int[])vector.get(0);
				vector.remove(0);
				actualTime = parsedID[0];	
			}
			
			switch (parsedID[1])
			{
				// HearEvent:
				case 0:	if (vector.size()>0)
						{ hearStringArray = (String[])vector.get(0); }
						refereeMsg();
						event[1] = 0; event[0] = actualTime;
						break;
				// VisualEvent:
				case 1: calcPositions();
						event[1] = 1; event[0] = actualTime;
						break;
				// BodyEvent:
				case 2: if (vector.size()>0)
						{ bodyDoubleArray = (double[])vector.get(0); }
						event[1] = 2; event[0] = actualTime;
						
						//player positions change
						if((actualTime>1500 && 1550>actualTime && playerChanged == false) || (actualTime>4500 && 4550>actualTime && playerChanged == true))
						{
							if(playerChanged)
								playerChanged = false;
							else
								playerChanged = true;
								
							switch(myNr)
							{
								case 1:					break;
								case 2:		myNr = 9;	break;
								case 3:		myNr = 10;	break;
								case 4:		myNr = 11;	break;
								case 5:		myNr = 8;	break;
								case 6:		myNr = 7;	break;
								case 7:		myNr = 6;	break;
								case 8:		myNr = 5;	break;
								case 9:		myNr = 2;	break;
								case 10:	myNr = 3;	break;
								case 11:	myNr = 4;	break;
								default: break;
							}
						}
						
						break;
				default: break;
			}
	}
	
    
} // end of class Worldmodel	

